////////////////////////////////////////////////////////////////////////////////
//  TMS470 Demo - Using the FLASH API Routines to In Circuit Program the FLASH
//
//  Program name : main_B1M_program_flash_01.c
//
//  This program uses the TI FLASH API routines to program the FLASH.
//  The program is loaded in to FLASH through the JTAG and copies API routines
//  into RAM. Then executes API routines to programs the FLASH with new data.
//
//  This program outputs the status data of the Flash API routines out the UART2
//  on the B1M EVM. The communication speed is 115,200 Baud.
//
//  Originator: John Mangino
//  Verified: Andreas Dannenberg
//  Texas Instruments, Inc
//  January 2006
//  Built with IAR Embedded Workbench Version: 4.30A
//  B1M EVM
//
////////////////////////////////////////////////////////////////////////////////

#define F05
#include "stdio.h"
#include "yfuns.h"
#include <intrinsic.h>
#include "iotms470r1b1m.h"
#include "tms470r1b1m_bit_definitions.h"
#include "flash470.h"
#include "f05.h"
#include "tms470_flash_definition.h"

// flash functions return value
#define FLASH_ERROR ( 0 )
#define FLASH_OK ( 1 )
//#define RAM_CFG          // Set up configuration with RAM at 0x00000000 and FLASH at 0x00400000
#define FLASH_CFG          // Set up configuration with RAM at 0x00400000 and FLASH at 0x00000000
//#define TEST               // Comenting this statement out removes the status outputs to the RS232 port

void SCI_Init(void);
void flash_01 (void);
static unsigned long FlashIdentify( void );
static unsigned char FlashUnlock( void);
static unsigned char FlashErase( UINT32 Delay, UINT32 FlashStartAddress,FLASH_CORE Bank,FLASH_SECT Sector  );
static unsigned char FlashCompact( UINT32 Delay, UINT32 FlashStartAddress,FLASH_CORE Bank,FLASH_SECT Sector  );
static unsigned char FlashProgram( UINT32 Delay, UINT32 * program_one ,UINT32 program_one_length, UINT32 FlashStartAddress,FLASH_CORE Bank,FLASH_SECT Sector );
static unsigned char FlashVerify( UINT32 * prog ,UINT32 length, UINT32 FlashStartAddress,FLASH_CORE Bank,FLASH_SECT Sector);
static unsigned char FlashCalcPSA(UINT32 length, UINT32 stadd1, UINT32 PSA_Value,FLASH_CORE Bank);
static unsigned char FlashPSAVerify (UINT32 length, UINT32 stadd1, UINT32 PSA_Value,FLASH_CORE Bank);
void copy_to_ram(void);

UINT32 PSAValue;
unsigned long DevID;
unsigned char Count;
UINT32 Delay;
UINT32 program_one_length;
UINT32 FlashStartAddress;
UINT16 Flash_Version;

void main(void)
{
  PCR = CLKDIV_1;                        // ICLK = SYSCLK/2
  PCR |= PENABLE;                        // enable peripherals
  GCR =  ZPLL_CLK_DIV_PRE_4;             // On B1M Development board 7.5 Mhz crystal * 8 / 4 = 14 Mhz
                                         // for flash programming
//  GCR =  ZPLL_MULT4 + ZPLL_CLK_DIV_PRE_4;// On A256 Development board 12.0 Mhz crystal * 4 / 4 = 12 Mhz
                                         // for flash programming

  SCI_Init();                            // Init SCI module
  DevID = FlashIdentify() ;              // Get device ID

  // Determine the memory structure for the specific part
  for( Count = 0 ; Count < NUM_OF_FLASH_DEVICES ; Count++ )
  {
    if( FlashDevices[Count]->DeviceID == DevID )
    {
      // set current flash device
      FlashDevice = FlashDevices[Count];
      break ;
    }
  }

  // Set the delay factor for the specified EVM by device
  switch (DevID & 0xfff)                        //
  {                                             //
    case 0x00000857 :                           // A256 with a 12 MHz Crystal PLL need to be divide by one
      Delay = 6;
      break;
    case 0x00000A5F :                           // B1M with 7.5 MHz PLL needs mul by 8 divide by 4
      Delay = 8;
      break;
  }

  copy_to_ram();
  flash_01();

  while (1);
}
//------------------------------------------------------------------------------
// This module copies API from FLASH to RAM
//------------------------------------------------------------------------------

// Make segment names accessible through __segment_begin /
// __segment_end functions
#pragma segment="API_SEGMENT"
#pragma segment="API_SEGMENT_D"

void copy_to_ram(void)
{
  int *i, *j;
  for(i = (int*)__segment_begin("API_SEGMENT"),
      j = (int*)__segment_begin("API_SEGMENT_D");
      i < (int*)__segment_end("API_SEGMENT");
      *i++ = *j++);
}

//------------------------------------------------------------------------------
// This module invokes the API Routines
//------------------------------------------------------------------------------
#pragma location="API_SEGMENT"
void flash_01(void)
{
  FLASH_CORE Flash_Bank;                      // Link Bank to Bank structure
  FLASH_SECT Flash_Sector;                    // Link sector to Sector structure

  int program_one[] =
  {
    0xaaaa,0xbbbb,0xcccc,0xdddd,0xeeee,0x1111,0x0020,0x0040,0x0080,0x0100,
    0x0200,0x0400,0x0800,0x1000,0x2000,0x4000,0x8000, 0xffff
  };

  SYSECR=0x00004007;
  Flash_Bank = FLASH_CORE0;                   // Bank = 0
  Flash_Sector = FLASH_SECT5;                 // Sector = 5
  FlashStartAddress = 0x00050000;
  program_one_length = 0x06;
  MFBALR0 &= ~RONLY;                          // Make FLASH Writable

  FlashUnlock();                              // Unlock Flash Keys
  FlashCompact((UINT32) Delay, (UINT32) FlashStartAddress,Flash_Bank,Flash_Sector);                                                                            // Flash Compaction
  FlashErase((UINT32) Delay, (UINT32) FlashStartAddress, Flash_Bank,Flash_Sector );       // Erase Flash
  FlashProgram((UINT32) Delay, (UINT32 *)program_one, program_one_length, (UINT32) FlashStartAddress,Flash_Bank,Flash_Sector);    // Program sector
  FlashVerify((UINT32 *)program_one, program_one_length, (UINT32) FlashStartAddress,Flash_Bank,Flash_Sector);                     // Verify Program
  FlashCalcPSA(program_one_length, (UINT32) FlashStartAddress,PSAValue,Flash_Bank);       // Generate PSA
  FlashPSAVerify(program_one_length, (UINT32) FlashStartAddress,PSAValue,Flash_Bank);     // Verify PSA
  Flash_Version = __Flash_API_Version_U16();

#ifdef TEST
  printf("\r\n Flash Version      0x%x ", Flash_Version);
#endif
}

//------------------------------------------------------------------------------
// SCI2 UART based character output function to be used by __write().
//------------------------------------------------------------------------------
#pragma location="API_SEGMENT"
int MyLowLevelPutchar(int x)
{
  while (!(SCI2CTL2 & TXRDY));                  // TX buffer ready?
  SCI2TXBUF = x;                                // Transmit character
  return x;                                     // Indicate "no error"
}
//------------------------------------------------------------------------------
// SCI2 Initialization
//
// Configures the SCI2 UART module for transmit operation using a data
// rate of 115,200bps.
//------------------------------------------------------------------------------
//#pragma location="API_SEGMENT"
void SCI_Init(void)
{
  SCI2CTL3 &= ~SW_NRESET;                       // Reset SCI2 state machine
  SCI2CCR = TIMING_MODE_ASYNC + CHAR_8;         // Async, 8-bit Char
  SCI2CTL2 |= TXENA;                            // TX enabled
  SCI2CTL3 |= CLOCK;                            // Internal clock
  SCI2LBAUD = 15;                               // 14MHz/8/115,200-1=12
  SCI2PC3 |= TX_FUNC;                           // SCITX is the SCI transmit pin
  SCI2CTL3 |= SW_NRESET;                        // Configure SCI2 state machine
}

//------------------------------------------------------------------------------
// Provide custom low-level __write() function to be used for console I/O.
// This function is based on the IAR template file <...>\arm\src\lib\write.c
//------------------------------------------------------------------------------
#pragma location="API_SEGMENT"
size_t __write(int handle, const unsigned char * buffer, size_t size)
{
  size_t nChars = 0;

  if (buffer == 0)
  {
     // This means that we should flush internal buffers.  Since we
     // don't we just return.  (Remember, "handle" == -1 means that all
     // handles should be flushed.)
    return 0;
  }

  // This template only writes to "standard out" and "standard err",
  // for all other file handles it returns failure.
  if (handle != _LLIO_STDOUT && handle != _LLIO_STDERR)
  {
    return _LLIO_ERROR;
  }

  for (/* Empty */; size != 0; --size)
  {
    if (MyLowLevelPutchar(*buffer++) < 0)
    {
      return _LLIO_ERROR;
    }
    ++nChars;
  }
  return nChars;
}

//------------------------------------------------------------------------------
// Read the Flash ID
//------------------------------------------------------------------------------
static unsigned long FlashIdentify( void )
{
  unsigned short ManID ;
  unsigned short DevID ;

  ManID = 0 ;
  DevID = DEV & 0x0FFF ;                    // mask away version info

  return( ( ( unsigned long )ManID << 16 ) | DevID ) ;
}

//------------------------------------------------------------------------------
// Condition the Flash memory
//
// This routine compacts the flash for the entire part.
//
//  static unsigned char FlashCompact( UINT32 Delay,   // Flash Delay factor
//                                    )
//
// Returns Success or Error
//------------------------------------------------------------------------------
#pragma location="API_SEGMENT"
static unsigned char FlashCompact( UINT32 Delay, UINT32 stadd,FLASH_CORE Bank,FLASH_SECT Sector)
{
  FLASH_STATUS_ST op;                   // Link op to the Flash Status structure

#ifdef TEST
  printf("\r\n\r\n    *** TMS470 Flash Compact Data ***");
#endif
  if (Flash_Compact_B((UINT32 *)stadd,Bank, Sector,(UINT32) Delay, ( FLASH_ARRAY_ST )0xFFE88000UL,&op)==1)
  {
#ifdef TEST
    printf("\r\n\r\n Bank = %i Sector = %i ",Bank,Sector);
    printf("\r\n Total number of compaction pulses       %i ", op.stat3);
    printf("\r\n Max number of compaction for any stick  %i ", op.stat4);
    printf("\r\n MSTAT Register                          0x%x ", op.stat2);
#endif
    return( FLASH_OK );
  }
  else
  {
#ifdef TEST
    printf("\r\n Compaction Fail");
#endif
    return( FLASH_ERROR ) ;
  }
}

//------------------------------------------------------------------------------
// Match the Flash protection keys
//
// Returns Success or Error
//------------------------------------------------------------------------------
#pragma location="API_SEGMENT"
static unsigned char FlashUnlock( void)
{
  UINT32 Keys[] = {0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF};

#ifdef FLASH_CFG
  Flash_Match_Key_B((UINT32 *)0x0000fff0UL,(UINT32 *)Keys,( FLASH_ARRAY_ST )0xFFE88000UL);
#endif
  return( FLASH_OK );
}

//------------------------------------------------------------------------------
// Erase the Flash
// static unsigned char FlashErase( UINT32 Delay,    // Flash Delay factor
//                                  UINT32 stadd,    // Starting address
//                                  FLASH_CORE Bank, // Bank
//                                  FLASH_SECT Sector// Sector
//                                  )
//
// Returns Success or Error
//------------------------------------------------------------------------------
#pragma location="API_SEGMENT"
static unsigned char FlashErase( UINT32 Delay, UINT32 stadd,FLASH_CORE Bank,FLASH_SECT Sector)
{
  FLASH_STATUS_ST op;                   // Link op to the Flash Status structure

  if( Flash_Blank_B((UINT32 *)stadd, FlashDevice->SectorLengthTable[Sector], Bank,( FLASH_ARRAY_ST )0xFFE88000UL,&op)==1)
  {
    op.stat1=0x12345678;
  }
  else
  {
    op.stat1=0x00000000;
#ifdef TEST
         printf("\r\n\r\n Bank = %i Sector = %i ",Bank,Sector);
         printf("\r\n First Non blank address      0x%x ", op.stat1);
         printf("\r\n First Non blank data         0x%x ", op.stat2);
         printf("\r\n Mode                         0x%x ", op.stat4);
#endif
   }

  if ( Flash_Erase_B((UINT32 *)stadd,0,Bank ,Sector,(UINT32) Delay, ( FLASH_ARRAY_ST )0xFFE88000UL,&op)==1)
  {
#ifdef TEST
    printf("\r\n\r\n Bank = %i Sector = %i ",Bank,Sector);
    printf("\r\n Total number of compaction pulses      %i ", op.stat4);
    printf("\r\n Max number of compaction for any stick %i ", op.stat1);
    printf("\r\n Flash Status Register                  %i ", op.stat2);
    printf("\r\n Number of pulses to erase the sector   %i ", op.stat3);
#endif
  }
  else
  {
#ifdef TEST
    printf("\r\n Erase Fail");
#endif
    return( FLASH_ERROR ) ;
  }
  return( FLASH_OK ) ;
}

//------------------------------------------------------------------------------
// Program the flash
// static unsigned char FlashProgram( UINT32 Delay,    // Flash Delay factor
//                                    UINT32 * prog ,  // Pointer to Buffer or program start
//                                    UINT32 length,   // Program length
//                                    UINT32 stadd,    // Starting address
//                                    FLASH_CORE Bank, // Bank
//                                    FLASH_SECT Sector// Sector
//                                  )
//
// Returns Success or Error
//------------------------------------------------------------------------------
#pragma location="API_SEGMENT"
static unsigned char FlashProgram( UINT32 Delay, UINT32 * prog ,UINT32 length, UINT32 stadd,FLASH_CORE Bank,FLASH_SECT Sector)
{
  FLASH_STATUS_ST op;                   // Link op to the Flash Status structure

  if (Flash_Prog_B((UINT32 *)stadd,prog, length,Bank,Delay,( FLASH_ARRAY_ST )0xFFE88000UL,&op) == 1)
  {
#ifdef TEST
    printf("\r\n\r\n Flash Programmed Sucessfully ");
    printf("\r\n Total Number of pulses           %i ", op.stat3);
    printf("\r\n Max Pulses to single location    %i ", op.stat4);
#endif
  }
  else
  {
#ifdef TEST
    printf("\r\n\r\n Program Fail");
    printf("\r\n Flash Fail at Location      0x%x ", op.stat1);
    printf("\r\n Flash Fail data at Location 0x%x ", op.stat2);
    printf("\r\n Pulses to single location   %i ", op.stat3);
    printf("\r\n Total Number of pulses      %i ", op.stat4);
#endif
    return( FLASH_ERROR ) ;
  }
  return( FLASH_OK ) ;
}

//------------------------------------------------------------------------------
// Verify the Flash
// static unsigned char FlashVerify( UINT32 * prog ,   // Pointer to Buffer or program start
//                                    UINT32 length,   // Program length
//                                    UINT32 stadd,    // Starting address
//                                    FLASH_CORE Bank, // Bank
//                                    FLASH_SECT Sector// Sector
//                                  )
//
// Returns Success or Error
//------------------------------------------------------------------------------
#pragma location="API_SEGMENT"
static unsigned char FlashVerify( UINT32 * prog ,UINT32 length, UINT32 stadd,FLASH_CORE Bank,FLASH_SECT Sector)
{
  FLASH_STATUS_ST op;                   // Link op to the Flash Status structure

  if (Flash_Verify_B((UINT32 *)stadd,prog, length,Bank,( FLASH_ARRAY_ST )0xFFE88000UL,&op) ==1)
  {
#ifdef TEST
    printf("\r\n Flash Verified Sucessfully ");
#endif
  }
  else
  {
#ifdef TEST
    printf("\r\n\r\n Verify Fail");
    printf("\r\n\r\n Bank = %i Sector = %i ",Bank,Sector);
    printf("\r\n Verify Fail at              0x%x ", op.stat1);
    printf("\r\n Verify Fail data            0x%x ", op.stat2);
    printf("\r\n Expected Data               0x%x ", op.stat3);
    printf("\r\n See documentation           0x%x ", op.stat4);
#endif
    return( FLASH_ERROR ) ;
  }
  return( FLASH_OK ) ;
}

//------------------------------------------------------------------------------
// Calculate the PSA of the program in the Flash
// static unsigned char FlashCalcPSA( UINT32 length,         // Program length
//                                    UINT32 stadd,          // Starting address
//                                    UINT32 Calc_PSA_Value, // Calculated PSA Value
//                                    FLASH_CORE Bank        // Bank
//                                  )
//
// Returns Success or Error
//------------------------------------------------------------------------------
#pragma location="API_SEGMENT"
static unsigned char   FlashCalcPSA(UINT32 length, UINT32 stadd, UINT32 Calc_PSA_Value,FLASH_CORE Bank)
{
  Calc_PSA_Value = Flash_PSA_Calc_U32((UINT32 *)stadd, length,0x00000000, 0x00000000,Bank,( FLASH_ARRAY_ST )0xFFE88000UL);
  PSAValue=Calc_PSA_Value;

#ifdef TEST
  printf("\r\n Calculated PSA Value 0x%x",Calc_PSA_Value );
#endif

  return( FLASH_OK ) ;
}

//------------------------------------------------------------------------------
// Verify the PSA value of the program in the Flash
// static unsigned char FlashPSAVerify( UINT32 length,         // Program length
//                                      UINT32 stadd1,         // Starting address
//                                      UINT32 Calc_PSA_Value, // Calculated PSA Value
//                                      FLASH_CORE Bank        // Bank
//                                    )
//
// Returns Success or Error
//------------------------------------------------------------------------------
#pragma location="API_SEGMENT"
static unsigned char FlashPSAVerify (UINT32 length, UINT32 stadd, UINT32 PSA_Value,FLASH_CORE Bank)
{
  FLASH_STATUS_ST op;                   // Link op to the Flash Status structure

  if (Flash_PSA_Verify_B((UINT32 *)stadd, length,PSA_Value,Bank,( FLASH_ARRAY_ST )0xFFE88000UL,&op)==1)
  {
#ifdef TEST
    printf("\r\n Flash PSA Verified Sucessfully ");
#endif
  }
  else
  {
#ifdef TEST
    printf("\r\n\r\n PSA Verify Fail");
    printf("\r\n PSA Failed Value Read Margin 0  0x%x ", op.stat1);
    printf("\r\n PSA Failed Value Read Margin 1  0x%x ", op.stat2);
    printf("\r\n PSA Failed Value Normal         0x%x ", op.stat3);
#endif
    return( FLASH_ERROR ) ;
  }

  return( FLASH_OK ) ;
}

